#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

typedef struct HidReport
{
    /*
     * +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
     * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  0        |                 Light (L)                     |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  1        |            UNUSED           | Btn | Light (H) |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     *
     */
    //uint16_t light:10;
        uint8_t  button1:1;
	uint8_t  button2:1;
        uint8_t  button3:1;
	uint8_t  button4:1;
	uint8_t  button5:1;
        uint8_t  button6:1;
	uint8_t  button7:1;
	uint8_t  button8:1;

} HidReport_t;

static HidReport_t reportBuffer;
static uchar idleRate; // Unused

/////////////////////////////////////////////////////////////////////
// USB report descriptor, generated by HID Descriptor Tool from
//    http://www.usb.org/developers/hidpage/dt2_4.zip
/////////////////////////////////////////////////////////////////////
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {

    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x05,                    // USAGE (Game Pad)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x07,                    //   USAGE_MAXIMUM (Button 8)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

///////////////////////////////////////////////////////////
uint16_t read_adc(uint8_t channel)
{
    uint16_t adc_val;

    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    adc_val = ADCL + ADCH*256;
    return adc_val;
}

uint16_t diff(uint8_t channel){
	uint16_t in1 = read_adc(channel);
	uint16_t in2 = read_adc(channel);
	if( in1 >= in2) return in1-in2;
	else return in2-in1;
}

///////////////////////////////////////////////////////////
void buildReport()
{
    reportBuffer.button1 = ( diff(PC4) > 75 );
    reportBuffer.button2 = ( diff(PC0) > 25 );
    reportBuffer.button3 = ( diff(PC1) > 25 );
    reportBuffer.button4 = ( diff(PC2) > 25 );
    //reportBuffer.button5 = ((PINB & (1<<PD3)) == 0);
    //reportBuffer.button6 = ((PINB & (1<<PD0)) == 0);
    //reportBuffer.button7 = ((PINB & (1<<PD2)) == 0);
    //reportBuffer.button8 = ((PINB & (1<<PD4)) == 0);
    //reportBuffer.light = read_adc(PC4);
}

///////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }

    return 0;
}

///////////////////////////////////////////////////////////
int main(void)
{
    //DDRC  = (1<<PC1); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = 0b00011101;                 // Enable pull-up at PC3
	
    //DDRB  = (1<<PB1);
    //PORTB = 0b00011101;
    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        /* always have report ready */
        buildReport();

        usbPoll();

        /* let the host know without asking */
        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
