#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_GET_MEAN_SOUND    0
#define RQ_GET_MAX_SOUND    1
#define RQ_GET_DIFF 2

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
uint16_t read_adc(uint8_t channel){
	ADMUX = 0b01000000;
	ADMUX |= channel;
	ADCSRA = 0b11000110;
	while((ADCSRA & (1<<ADSC)));
	return ADCL+ADCH*256;

}
static uint16_t sound;
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if (rq->bRequest == RQ_GET_MEAN_SOUND)
    {
		int i;
		sound=0;
		for(i=0;i<100;i++)
        	sound += read_adc(rq->wValue.bytes[0]);
		sound /=100;
        usbMsgPtr = (uchar*) &sound;

        return sizeof(sound);
    }
	else if (rq->bRequest == RQ_GET_MAX_SOUND)
    {
		int i;
		int max=0;
		for(i=0;i<100;i++){
        	sound = read_adc(rq->wValue.bytes[0]);
			if(max < sound){
				max=sound;
			}
		}
		sound = max;
        usbMsgPtr = (uchar*) &sound;
        return sizeof(sound);
    }
	else if (rq->bRequest == RQ_GET_DIFF)
    {
		uint16_t in1 = read_adc(rq->wValue.bytes[0]);
		uint16_t in2 = read_adc(rq->wValue.bytes[0]);
		if(in1 > in2) 
			sound = in1-in2;
		else
			sound = in2-in1;

        usbMsgPtr = (uchar*) &sound;
        return sizeof(sound);
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    //DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
//    PORTC = (1<<PC4)|(1<<PC0)|(1<<PC1)|(1<<PC2);                 // Enable pull-up at PC3

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
